/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
'use strict';
define(["require", "exports", 'monaco', './features/extraInfoSupport', './features/commentsSupport', './features/declarationSupport', './features/occurrencesSupport', './features/referenceSupport', './features/outlineSupport', './features/parameterHintsSupport', './features/renameSupport', './features/formattingSupport', './features/bufferSyncSupport', './features/suggestSupport', './features/configuration', './features/navigateTypesSupport', './typescriptServiceClient'], function (require, exports, monaco, ExtraInfoSupport, CommentsSupport, DeclarationSupport, OccurrencesSupport, ReferenceSupport, OutlineSupport, ParameterHintsSupport, RenameSupport, FormattingSupport, BufferSyncSupport, SuggestSupport, Configuration, NavigateTypeSupport, TypeScriptServiceClient) {
    function activate() {
        var MODE_ID_TS = 'typescript';
        var MODE_ID_TSX = 'typescriptreact';
        var MY_PLUGIN_ID = 'vs.language.typescript';
        var clientHost = new TypeScriptServiceClientHost(monaco.Services.MarkerService, monaco.Services.ConfigurationService);
        var client = clientHost.serviceClient;
        // Register the supports for both TS and TSX so that we can have separate grammars but share the mode
        registerSupports(MODE_ID_TS, client);
        registerSupports(MODE_ID_TSX, client);
        return null;
    }
    exports.activate = activate;
    function registerSupports(modeID, client) {
        monaco.Modes.TokenTypeClassificationSupport.register(modeID, {
            wordDefinition: /(-?\d*\.\d\w*)|([^\`\~\!\@\#\%\^\&\*\(\)\-\=\+\[\{\]\}\\\|\;\:\'\"\,\.\<\>\/\?\s]+)/g
        });
        monaco.Modes.ElectricCharacterSupport.register(modeID, {
            brackets: [
                { tokenType: 'delimiter.curly.ts', open: '{', close: '}', isElectric: true },
                { tokenType: 'delimiter.square.ts', open: '[', close: ']', isElectric: true },
                { tokenType: 'delimiter.paren.ts', open: '(', close: ')', isElectric: true }
            ],
            docComment: { scope: 'comment.documentation', open: '/**', lineStart: ' * ', close: ' */' }
        });
        monaco.Modes.CharacterPairSupport.register(modeID, {
            autoClosingPairs: [
                { open: '{', close: '}' },
                { open: '[', close: ']' },
                { open: '(', close: ')' },
                { open: '"', close: '"', notIn: ['string'] },
                { open: '\'', close: '\'', notIn: ['string', 'comment'] }
            ]
        });
        monaco.Modes.ExtraInfoSupport.register(modeID, new ExtraInfoSupport(client));
        monaco.Modes.CommentsSupport.register(modeID, new CommentsSupport());
        monaco.Modes.DeclarationSupport.register(modeID, new DeclarationSupport(client));
        monaco.Modes.OccurrencesSupport.register(modeID, new OccurrencesSupport(client));
        monaco.Modes.ReferenceSupport.register(modeID, new ReferenceSupport(client));
        monaco.Modes.OutlineSupport.register(modeID, new OutlineSupport(client));
        monaco.Modes.ParameterHintsSupport.register(modeID, new ParameterHintsSupport(client));
        monaco.Modes.RenameSupport.register(modeID, new RenameSupport(monaco.Services.ModelService, client));
        monaco.Modes.FormattingSupport.register(modeID, new FormattingSupport(monaco.Services.ModelService, client));
        monaco.Modes.NavigateTypesSupport.register(modeID, new NavigateTypeSupport(monaco.Services.ModelService, client, modeID));
        monaco.Modes.OnEnterSupport.register(modeID, {
            brackets: [
                { open: '{', close: '}' },
                { open: '[', close: ']' },
                { open: '(', close: ')' },
            ],
            regExpRules: [
                {
                    // e.g. /** | */
                    beforeText: /^\s*\/\*\*(?!\/)([^\*]|\*(?!\/))*$/,
                    afterText: /^\s*\*\/$/,
                    action: { indentAction: monaco.Modes.IndentAction.IndentOutdent, appendText: ' * ' }
                },
                {
                    // e.g. /** ...|
                    beforeText: /^\s*\/\*\*(?!\/)([^\*]|\*(?!\/))*$/,
                    action: { indentAction: monaco.Modes.IndentAction.None, appendText: ' * ' }
                },
                {
                    // e.g.  * ...|
                    beforeText: /^(\t|(\ \ ))*\ \*\ ([^\*]|\*(?!\/))*$/,
                    action: { indentAction: monaco.Modes.IndentAction.None, appendText: '* ' }
                },
                {
                    // e.g.  */|
                    beforeText: /^(\t|(\ \ ))*\ \*\/\s*$/,
                    action: { indentAction: monaco.Modes.IndentAction.None, removeText: 1 }
                }
            ]
        });
        new BufferSyncSupport(monaco.Services.ModelService, monaco.Services.MarkerService, client, modeID);
        // Register suggest support as soon as possible and load configuration lazily
        // TODO: Eventually support eventing on the configuration service & adopt here
        var suggestSupport = new SuggestSupport(monaco.Services.ModelService, client);
        monaco.Modes.SuggestSupport.register(modeID, suggestSupport);
        Configuration.load(modeID, monaco.Services.ConfigurationService).then(function (config) {
            suggestSupport.setConfiguration(config);
        });
    }
    var TypeScriptServiceClientHost = (function () {
        function TypeScriptServiceClientHost(markerService, configService) {
            this.markerService = markerService;
            this.client = new TypeScriptServiceClient(this, monaco.Services.ConfigurationService);
            this.syntaxDiagnostics = Object.create(null);
        }
        Object.defineProperty(TypeScriptServiceClientHost.prototype, "serviceClient", {
            get: function () {
                return this.client;
            },
            enumerable: true,
            configurable: true
        });
        /* internal */ TypeScriptServiceClientHost.prototype.syntaxDiagnosticsReceived = function (event) {
            var body = event.body;
            if (body.diagnostics) {
                var markers = this.createMarkerDatas(body.diagnostics);
                this.syntaxDiagnostics[body.file] = markers;
            }
        };
        /* internal */ TypeScriptServiceClientHost.prototype.semanticDiagnosticsReceived = function (event) {
            var body = event.body;
            if (body.diagnostics) {
                var markers = this.createMarkerDatas(body.diagnostics);
                var syntaxMarkers = this.syntaxDiagnostics[body.file];
                if (syntaxMarkers) {
                    delete this.syntaxDiagnostics[body.file];
                    markers = syntaxMarkers.concat(markers);
                }
                this.markerService.changeOne('typescript', this.serviceClient.asUrl(body.file), markers);
            }
        };
        TypeScriptServiceClientHost.prototype.createMarkerDatas = function (diagnostics) {
            var markers = [];
            for (var i = 0; i < diagnostics.length; i++) {
                var diagnostic = diagnostics[i];
                var marker = {
                    severity: monaco.Services.Severity.Error,
                    message: diagnostic.text,
                    startLineNumber: diagnostic.start.line,
                    startColumn: diagnostic.start.offset,
                    endLineNumber: diagnostic.end.line,
                    endColumn: diagnostic.end.offset
                };
                markers.push(marker);
            }
            return markers;
        };
        return TypeScriptServiceClientHost;
    })();
});
//# sourceMappingURL=typescriptMain.js.map